@@ -19,15 +19,17 @@ import os
 import re
 import json
 import datetime
-
-from pymatgen.core.structure import Structure
-from pymatgen.transformations.standard_transformations import transformation_from_dict
-from pymatgen.io.cifio import CifParser
-from pymatgen.io.vaspio import Poscar
 from copy import deepcopy
 
+from pymatgen.core.structure import Structure
+from pymatgen.transformations.transformation_abc import AbstractTransformation
 
-class TransformedStructure(object):
+from pymatgen.io.cifio import CifParser
+from pymatgen.io.vaspio import Poscar
+from pymatgen.serializers.json_coders import MSONable
+
+
+class TransformedStructure(MSONable):
     """
     Container object for new structures that include history of transformations.
     
@@ -66,7 +68,7 @@ class TransformedStructure(object):
             self._source = history[0]
             for i in xrange(1, len(history)):
                 self._structures.append(Structure.from_dict(history[i]['input_structure']))
-                self._transformations.append(transformation_from_dict(history[i]))
+                self._transformations.append(AbstractTransformation.from_dict(history[i]))
                 self._transformation_parameters.append(history[i].get('output_parameters', {}))
 
         self._structures.append(structure)
@@ -131,12 +133,13 @@ class TransformedStructure(object):
 
         if return_alternatives and transformation.is_one_to_many:
             starting_struct = self._structures[-1]
-            ranked_list = transformation.apply_transformation(starting_struct, return_ranked_list=return_alternatives)
+            ranked_list = transformation.apply_transformation(starting_struct,
+                                        return_ranked_list=return_alternatives)
             #generate the alternative structures
             alts = []
             for x in ranked_list[1:]:
                 struct = x.pop('structure')
-                other_paras = [p for p in self._other_parameters]
+                other_paras = self._other_parameters.copy()
                 hist = self.history
                 actual_transformation = x.pop('transformation', transformation)
                 tdict = actual_transformation.to_dict
@@ -179,8 +182,8 @@ class TransformedStructure(object):
                 vasp input files from structures
             generate_potcar:
                 Set to False to generate a POTCAR.spec file instead of a POTCAR,
-                which contains the POTCAR labels but not the actual POTCAR. Defaults
-                to True.
+                which contains the POTCAR labels but not the actual POTCAR.
+                Defaults to True.
         """
         d = vasp_input_set.get_all_vasp_input(self._structures[-1], generate_potcar)
         d['transformations.json'] = json.dumps(self.to_dict)
@@ -213,7 +216,7 @@ class TransformedStructure(object):
         output.append("\nTransformation history")
         output.append("------------")
         for i, t in enumerate(self._transformations):
-            output.append(str(t.to_dict) + ' ' + str(self._transformation_parameters[i]))
+            output.append("{} {}".format(t.to_dict, self._transformation_parameters[i]))
         output.append("\nOther parameters")
         output.append("------------")
         output.append(str(self._other_parameters))
@@ -291,7 +294,8 @@ class TransformedStructure(object):
         return d
 
     @staticmethod
-    def from_cif_string(cif_string, transformations=[], primitive=True):
+    def from_cif_string(cif_string, transformations=[], primitive=True,
+                        occupancy_tolerance=1.):
         """
         Generates TransformedStructure from a cif string.
 
@@ -300,15 +304,19 @@ class TransformedStructure(object):
                 Input cif string. Should contain only one structure. For cifs
                 containing multiple structures, please use CifTransmuter.
             transformations:
-                Sequence of transformations to be applied to the input structure.
-            primitive:
-                Option to set if the primitive cell should be extracted. Defaults
-                to True. However, there are certain instances where you might want
-                to use a non-primitive cell, e.g., if you are trying to generate
-                all possible orderings of partial removals or order a disordered
+                Sequence of transformations to be applied to the input
                 structure.
+            primitive:
+                Option to set if the primitive cell should be extracted.
+                Defaults to True. However, there are certain instances where
+                you might want to use a non-primitive cell, e.g., if you are
+                trying to generate all possible orderings of partial removals
+                or order a disordered structure.
+            occupancy_tolerance:
+                If total occupancy of a site is between 1 and
+                occupancy_tolerance, the occupancies will be scaled down to 1.
         """
-        parser = CifParser.from_string(cif_string)
+        parser = CifParser.from_string(cif_string, occupancy_tolerance)
         raw_string = re.sub("'", "\"", cif_string)
         cif_dict = parser.to_dict
         cif_keys = cif_dict.keys()
@@ -334,7 +342,7 @@ class TransformedStructure(object):
         if not p.true_names:
             raise ValueError("Transformation can be craeted only from POSCAR strings with proper VASP5 element symbols.")
         raw_string = re.sub("'", "\"", poscar_string)
-        s = p.struct
+        s = p.structure
         source_info = {'source': "uploaded POSCAR", 'datetime':str(datetime.datetime.now()), 'original_file':raw_string}
         return TransformedStructure(s, transformations, [source_info])
 
