@@ -14,7 +14,7 @@ __email__ = "shyue@mit.edu"
 __date__ = "Mar 13, 2012"
 
 import math
-
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 
 
@@ -28,6 +28,9 @@ def get_publication_quality_plot(width=8, height=None):
         height.
             Height of plot in inches. Defaults to width * golden ratio.
     """
+
+    mpl.rcParams['font.serif'] = 'Times New Roman'
+    mpl.rcParams['font.sans-serif'] = 'Arial'
     golden_ratio = (math.sqrt(5) - 1.0) / 2.0
     if not height:
         height = int(width * golden_ratio)
